#ifndef SPU_H
#define SPU_H
//-------------------------------------------------------
#include <psone/gpu.h>
//-------------------------------------------------------
typedef struct {
	u_short left;	// Left channel
	u_short right;	// Right channel
} SndVolume;

typedef struct{
	u_long id;			// should be VAGp
	u_long version;		// should be 0x00000020
	u_long reserved_0;
	u_long datalen;		// length in bytes of sample data
	u_long freq;		// frequency
	u_long reserved_1[3];
	u_long name[4];	// name of sample
}VAGHEADER;

typedef struct {
	u_char *mod_start;			// start of MOD in memory
	u_char volume;				// MOD master volume (0-63)
	u_char tracks;				// number of tracks in mod
	u_char patterns;				// number of patterns in song
	u_long pattern_data_size;		// size of pattern data
	u_long sample_start;			// SRAM start address of samples
	u_long sample_end;			// SRAM end address of samples
	u_long *row_pointer;			// pointer to current row
	u_long sample_address[31];	// address in SB of each sample
	u_short key_on;
	u_char speed;				// ticks per row
	u_char tick;				// current tick
	u_char current_row;
	u_char current_order;
	u_char current_pattern;
	u_char song_length;			// no of orders in song
	u_char sample_volume[31];	// volume info for each sample
	char channel_volume[8];		// current channel vols
	u_char last_sample[8];		// last sample played on channel
	u_char effect[8];			// current effect active
	u_char LFO_pointer[8];
	u_char LFO_speed[8];
	u_char LFO_depth[8];
	u_short last_period[8];		// last pitch period on this chan
	char pitch_slide[8];			// amount to slide pitch by
	char volume_slide[8];			// amount to slide volume by
	u_char on;
} HITMOD;

//---- MOD playing routines -----------------------------
void InitMod(u_char num, u_char *modaddr);
void PollMod(u_char num);
void StartMod(u_char num);
void PauseMod(u_char num);
void ReStartMod(u_char num);
void SetModVol (u_char num, u_char mod_vol);
u_char GetModVol (u_char num);
//-------------------------------------------------------
void SsInit(void);
void SsStop(void);
//-------------------------------------------------------
void SsSetMVol (u_short vol_left, u_short vol_right);
void SsGetMVol (SndVolume * m_vol);
u_long SsGetVoiceMask (void);
//-------------------------------------------------------
void SPU_VoiceSetData(u_char voice, u_long sb_addr);
long SsVagLoad(VAGHEADER * vag);
char SsPlaySound(u_long addr, char voice, int lvol, int rvol);
//-------------------------------------------------------
#endif // SPU_H
