#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

typedef int BOOL;
#define TRUE 1
#define FALSE 0

static BOOL
BuildCueFile (char *rawFile, char *cueFile)
{
	FILE *fp;
	char buf[10 * 1024];

	if (!rawFile || !cueFile)
		return FALSE;

	fp = fopen (cueFile, "w");
	if (!fp)
	  {
		  printf ("ERROR: Failed to create cue file.\n");
		  return FALSE;
	  }

	(void) sprintf (buf, "FILE \"%s\" BINARY\n", rawFile);
	fputs (buf, fp);
	fputs (" TRACK 01 MODE2/2352\n INDEX 01 00:00:00\n", fp);

	fclose (fp);

	return TRUE;
}

static BOOL
Iso2Raw (char *isoFile, char *rawFile)
{
	FILE *fp1;
	FILE *fp2;
	unsigned char sec[16];
	unsigned char sub[8];
	unsigned char buf[2353];
	unsigned char edc[4];
	unsigned char ecc[280];
	unsigned int out;
	int c;
	int thesec = 0;
	char dog[10];

	(void) memset (sec, 0xFF, 16);
	sec[0] = 0;
	sec[11] = 0;
	sec[12] = 0;
	sec[13] = 2;
	sec[14] = 0;
	sec[15] = 2;

	(void) memset (sub, 0x00, 8);
	(void) memset (edc, 0x01, 4);
	(void) memset (ecc, 0x02, 280);

	fp1 = fopen (isoFile, "rb");

	if (!fp1)
	  {
		  printf ("ERROR: Failed to open ISO file for reading.\n");
		  return FALSE;
	  }

	fp2 = fopen (rawFile, "wb");

	if (!fp2)
	  {
		  printf ("ERROR: Failed to create raw ISO file.\n");
		  return FALSE;
	  }

	while (1)
	  {
		  c = fread (buf, sizeof (unsigned char), 2048, fp1);
		  if (!c)
			  break;

		  fwrite (sec, sizeof (unsigned char), 16, fp2);
		  fwrite (sub, sizeof (unsigned char), 8, fp2);
		  fwrite (buf, sizeof (unsigned char), 2048, fp2);
		  fwrite (edc, sizeof (unsigned char), 4, fp2);
		  fwrite (ecc, sizeof (unsigned char), 276, fp2);

		  thesec++;

		  if (thesec > 74)
		    {
			    thesec = 0;
			    sec[13] = sec[13] + 1;
		    }



		  (void) sprintf (dog, "0x%d", thesec);
		  (void) sscanf (dog, "%x", &out);

		  sec[14] = out;
	  }

	fclose (fp2);
	fclose (fp1);

	return TRUE;
}

static BOOL
Copybootinfo (char *BootInfo, char *rawFile)
{
	FILE *fp1;
	FILE *fp2;
	unsigned char buf[2352];
	int a = 0;
	struct stat sinfo;

	if (!BootInfo || !rawFile)
		return FALSE;

	fp1 = fopen (BootInfo, "rb");

	if (!fp1)
	  {
		  printf ("ERROR: Failed to open bootinfo file for reading.\n");
		  return FALSE;
	  }

	if (stat (BootInfo, &sinfo) != -1)
	  {
		  if (sinfo.st_size != (16 * 2352))
		    {
			    fclose (fp1);
			    printf ("ERROR: bootinfo file specified is the incorrect size.\n");
			    return FALSE;
		    }
	  }

	fp2 = fopen (rawFile, "r+b");

	if (!fp2)
	  {
		  printf ("ERROR: Failed to open raw ISO image file for writing.\n");
		  return FALSE;
	  }

	if (stat (rawFile, &sinfo) != -1)
	  {
		  div_t dinfo;
		  dinfo = div (sinfo.st_size, 2352);

		  if (dinfo.rem != 0)
		    {
			    fclose (fp1);
			    fclose (fp2);
			    printf ("ERROR: The raw ISO file is not a multiple of 2352 bytes.\n");
			    return FALSE;
		    }
	  }

	fseek (fp2, 0, SEEK_SET);

	while (1)
	  {
		  int c = fread (buf, sizeof (unsigned char), 2352, fp1);

		  if (!c)
			  break;

		  fseek (fp2, a, SEEK_SET);
		  fwrite (buf, sizeof (unsigned char), c, fp2);
		  a += c;
	  }

	fclose (fp1);
	fclose (fp2);

	return TRUE;
}

static BOOL
IsISOValid (char *isoFile)
{
	FILE *fp;
	unsigned char buf[128];
	int c;
	struct stat sinfo;

	fp = fopen (isoFile, "rb");

	if (!fp)
	  {
		  printf ("ERROR: Failed to open ISO image file.\n");
		  return FALSE;
	  }

	c = fread (buf, sizeof (unsigned char), 32, fp);
	if (c != 32)
	  {
		  fclose (fp);
		  printf ("ERROR: Invalid mkisofs ISO image, file too small.\n");
		  return FALSE;
	  }

	if (buf[0] == 0x00 && buf[1] == 0xFF && buf[2] == 0xFF
	    && buf[3] == 0xFF && buf[4] == 0xFF && buf[5] == 0xFF
	    && buf[6] == 0xFF && buf[7] == 0xFF && buf[8] == 0xFF
	    && buf[9] == 0xFF && buf[10] == 0xFF && buf[11] == 0x00
	    && buf[12] == 0x00 && buf[14] == 0x00)
	  {
		  fclose (fp);
		  printf ("ERROR: ISO image already has Raw Sector information.\n");
		  return FALSE;
	  }

	fclose (fp);

	if (stat (isoFile, &sinfo) != -1)
	  {
		  if ((sinfo.st_size % 2048) != 0)
		    {
			    printf ("ERROR: ISO file specified is not 2048 bytes per sector.\n");
			    return FALSE;
		    }
	  }

// XXX Would like to validate if the ISO has
// version numbers enabled
// and if it is mode1/mode2 etc etc...
// could even check if there is a system.cnf file!

	return TRUE;
}

static BOOL
PatchIso (char *BootInfo, char *rawFile)
{
	if (!Copybootinfo (BootInfo, rawFile))
		return FALSE;

	return TRUE;
}

int
main (int argc, char *argv[])
{
	char *BootInfo;
	char *isoFile;
	char *temp;
	char *infile;
	char *cueFile;
	char *rawFile;
	int bootinfo = 1;
	int n = 0;

	printf ("---------------------------------\n");
	printf ("ISO9660 2048->2352 converter tool\n");
	printf ("---------------------------------\n\n");

	printf("argc %d\n" , argc);

	if ( (argc - 1) < 2)
	{
		printf("Wrong number of argument.\n\tUsage: iso2raw bootinfo isofile.iso\n\t\tIf you don't want to merge a bootinfo file, write nobootinfo\n");
		return 0;
	}


	infile = malloc (1024);
	strcpy (infile, argv[2]);
	printf ("Input file: %s\n", infile);

	rawFile = malloc (1024);
	strcpy (rawFile, infile);
	
	while(rawFile[n] != '.')
	{
		if(n == strlen(rawFile))
		{
			printf("***ERROR*** Iso File don't have its .iso extenction in its pathname\n");
			exit(0);
		}
		n++;
	}

	n++;

	rawFile[n] = 'b';
	rawFile[n+1] = 'i';
	rawFile[n+2] = 'n';
	
	printf ("Output file: %s\n", rawFile);

	cueFile = malloc (1024);
	strcpy (cueFile, infile);

	n = 0;

	while(cueFile[n] != '.')
	{
		if(n == strlen(cueFile))
		{
			printf("***ERROR*** Iso File don't have its .iso extenction in its pathname\n");
			exit(0);
		}
		n++;
	}

	n++;

	cueFile[n] = 'c';
	cueFile[n+1] = 'u';
	cueFile[n+2] = 'e';



	printf ("Cue file: %s\n", cueFile);

	if (!strcmp (argv[1], "nobootinfo"))
		bootinfo = 0;
	else
	  {
		  BootInfo = argv[1];
		  printf ("Bootinfo file: %s\n", BootInfo);
	  }

	if (!IsISOValid (infile))
		exit (1);

	if (!Iso2Raw (infile, rawFile))
		exit (1);

	if (bootinfo)
		if (!PatchIso (BootInfo, rawFile))
			exit (1);

	if (!BuildCueFile (rawFile, cueFile))
		exit (1);

	printf ("Done.\n", cueFile);

return 0;
}
